"""
read Bemol data and cluster users based on the number of similar products they purchased
"""

import numpy
import logging
import sys
import gzip
import re
from apgl.graph import *
from apgl.generator import *
from apgl.sandbox.IterativeSpectralClustering import IterativeSpectralClustering
from apgl.sandbox.IterativeSpectralClustering import DatedPurchasesGraphListIterator
from apgl.sandbox.IterativeSpectralClustering import MyDictionary
from apgl.util.PathDefaults import PathDefaults

numpy.random.seed(21)
numpy.set_printoptions(suppress=True, precision=3, linewidth=200, threshold=40000)
logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)


def getBemolGraphIterator(dir):
    """
    Load the Bemol data from the directory dir
    """
    nb_kept_user = 1000 # keep the first ones
    #f_data_name = "idu__idp__semaine__annee.dat.gz"
    f_data_name = dir +  "idu__idp__semaine__annee__first_users1000.dat.gz"
    #f_data_name = "idu__idp__semaine__annee__first_users1000.dat.gz"

    #===========================================
    # read data
    purchasesList = []
    dict_user = MyDictionary()
    try:
            f_data = gzip.open(f_data_name, 'rb')
            #f_small_data = gzip.open("blop.dat.gz", 'wb')

            for line in f_data:
                    m = re.match("(\d+)\s(\d+)\s(\d+)\s(\d+)", line)
                    if dict_user.index(int(m.group(1))) < nb_kept_user:
                            purchasesList.append([int(m.group(1)), int(m.group(2)), int(m.group(3)), int(m.group(4))])
                        #f_small_data.write(line)
                        #print purchasesList[-1]

            graphIterator = DatedPurchasesGraphListIterator(purchasesList)

            return graphIterator
    except IOError:
        print 'cannot open', f_data_name
        raise 


def cluster():
    k1 = 20 # numCluster to learn
    k2 = 40 # numEigenVector kept

    dir = PathDefaults.getDataDir() + "cluster/"
    graphIterator = getBemolGraphIterator(dir)
    #===========================================
    # cluster
    print "compute clusters"
    clusterer = IterativeSpectralClustering(k1, k2)
    clustersList = clusterer.clusterFromIterator(graphIterator, True)

    for i in range(len(clustersList)):
              clusters = clustersList[i]
              print(clusters)

# to run
# python -c "execfile('apgl/clusterexp/BemolData.py')"

