"""
Compare our clustering method and that of Ning et al. on the Bemol data 
"""
import itertools
import sys
import logging
import numpy
from apgl.graph import *
from apgl.util.PathDefaults import PathDefaults
from apgl.sandbox.NingSpectralClustering import NingSpectralClustering
from apgl.sandbox.IterativeSpectralClustering import IterativeSpectralClustering, IncreasingSubgraphListIterator
from apgl.clusterexp.BemolData import getBemolGraphIterator

numpy.random.seed(21)
logging.basicConfig(stream=sys.stdout, level=logging.DEBUG)
numpy.set_printoptions(suppress=True, linewidth=60)

resultsDir = PathDefaults.getOutputDir() + "cluster/"
dataDir = PathDefaults.getDataDir() + "cluster/"
startingIteration = 30
endingIteration = 1000 # put a big number to have all iterations
iterator = itertools.islice(getBemolGraphIterator(dataDir),startingIteration, endingIteration)

numMethods = 3
numMeasures = 2 
numGraphs = 0
stepSize = 20

for i in itertools.islice(iterator, stepSize):
    numGraphs += 1 

modularities = numpy.zeros((numGraphs, numMethods))
kwayNormalisedCuts = numpy.zeros((numGraphs, numMethods))

k1 = 25
k2 = 8*k1

#Variables to choose which methods to run
runIASC = False
runExact = False
runNing = True

if runIASC or runExact:
    clusterer = IterativeSpectralClustering(k1, k2)
    clusterer.nb_iter_kmeans = 20

def recordResults(clusterList, timeList, fileName):
    iterator = itertools.islice(getBemolGraphIterator(dataDir),startingIteration, endingIteration)
    measures = numpy.zeros((numGraphs, numMeasures))

    for i in range(numGraphs):
        W = iterator.next()
        measures[i, 0] = GraphUtils.modularity(W, clusterList[i])
        measures[i, 1] = GraphUtils.kwayNormalisedCut(W, clusterList[i])

    file = open(fileName, 'w')
    numpy.savez(file, measures, timeList)
    logging.info("Saved file as " + resultsFileName)

if runIASC:
    logging.info("Running approximate method")
    iterator = itertools.islice(getBemolGraphIterator(dataDir),startingIteration, endingIteration)
    clusterList, timeList = clusterer.clusterFromIterator(iterator, timeIter=True)

    resultsFileName = resultsDir + "BemolResultsIASC.npz"
    recordResults(clusterList, timeList, resultsFileName)

if runExact:
    logging.info("Running exact method")
    iterator = itertools.islice(getBemolGraphIterator(dataDir),startingIteration, endingIteration)
    clusterList, timeList = clusterer.clusterFromIterator(iterator, False, timeIter=True)

    resultsFileName = resultsDir + "BemolResultsExact.npz"
    recordResults(clusterList, timeList, resultsFileName)

if runNing:
    logging.info("Running Nings method")
    iterator = itertools.islice(getBemolGraphIterator(dataDir),startingIteration, endingIteration)
    clusterer = NingSpectralClustering(k1)
    clusterList, timeList = clusterer.cluster(iterator, timeIter=True)

    resultsFileName = resultsDir + "BemolResultsNing.npz"
    recordResults(clusterList, timeList, resultsFileName)

# to run
# python -c "execfile('apgl/clusterexp/BemolExperiment.py')"


