
from apgl.graph import *
from apgl.graph.test.MatrixGraphTest import MatrixGraphTest
from apgl.util import * 
import unittest
import numpy
import scipy
import logging

class SparseGraphTest(unittest.TestCase, MatrixGraphTest):
    def setUp(self):
        self.GraphType = SparseGraph
        self.initialise()

    def testInit(self):
        numVertices = 0
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        graph = SparseGraph(vList)

        numVertices = 10
        numFeatures = 1
        vList = VertexList(numVertices, numFeatures)
        graph = SparseGraph(vList)
        self.assertEquals(graph.weightMatrixType(), scipy.sparse.csr_matrix)

        self.assertRaises(ValueError, SparseGraph, [])
        self.assertRaises(ValueError, SparseGraph, vList, 1)
        self.assertRaises(ValueError, SparseGraph, vList, True, 1)

        #Now test invalid values of W
        W = numpy.zeros((numVertices, numVertices))
        self.assertRaises(ValueError, SparseGraph, vList, True, W)

        W = scipy.sparse.lil_matrix((numVertices+1, numVertices))
        self.assertRaises(ValueError, SparseGraph, vList, True, W)

        W = scipy.sparse.lil_matrix((numVertices, numVertices))
        W[0, 1] = 1
        self.assertRaises(ValueError, SparseGraph, vList, True, W)

        W = scipy.sparse.lil_matrix((numVertices, numVertices))
        graph = SparseGraph(vList, W=W)

        self.assertEquals(graph.weightMatrixType(), scipy.sparse.lil_matrix)

    def testNativeAdjacencyMatrix(self):
        numVertices = 10 
        graph = SparseGraph(GeneralVertexList(numVertices))

        graph.addEdge(1, 1, 0.1)
        graph.addEdge(1, 3, 0.5)
        graph.addEdge(2, 5, 1)
        graph.addEdge(7, 0, 2)

        A = graph.nativeAdjacencyMatrix()
        self.assertEquals(A[0, 7], 1)
        self.assertEquals(A[7, 0], 1)
        self.assertEquals(A[1, 3], 1)
        self.assertEquals(A[3, 1], 1)
        self.assertEquals(A[1, 1], 1)
        self.assertEquals(A[2, 5], 1)
        self.assertEquals(A[5, 2], 1)
        self.assertEquals(A.getnnz(), 7)

        graph = SparseGraph(GeneralVertexList(numVertices), False)
        graph.addEdge(1, 1, 0.1)
        graph.addEdge(1, 3, 0.5)
        graph.addEdge(2, 5, 1)

        A = graph.nativeAdjacencyMatrix()
        self.assertEquals(A[1, 3], 1)
        self.assertEquals(A[1, 1], 1)
        self.assertEquals(A[2, 5], 1)
        self.assertEquals(A.getnnz(), 3)

    def testConcat(self):
        numVertices = 5
        graph = SparseGraph(GeneralVertexList(numVertices))
        graph.addEdge(1, 1, 0.1)
        graph.addEdge(1, 3, 0.5)
        graph.addEdge(2, 4, 1)
        graph.addEdge(2, 3, 2)
        graph.setVertex(0, "abc")

        graph2 = SparseGraph(GeneralVertexList(numVertices))
        graph2.addEdge(1, 1)
        graph2.addEdge(1, 4)
        graph2.setVertex(1, "def")

        graph3 = graph.concat(graph2)

        self.assertTrue(graph3.getNumVertices, 10)
        self.assertEquals(graph3.getVertex(0), "abc")
        self.assertEquals(graph3.getVertex(6), "def")
        self.assertEquals(graph3.getEdge(1, 1), 0.1)
        self.assertEquals(graph3.getEdge(1, 3), 0.5)
        self.assertEquals(graph3.getEdge(2, 4), 1)
        self.assertEquals(graph3.getEdge(2, 3), 2)

        self.assertEquals(graph3.getEdge(6, 6), 1)
        self.assertEquals(graph3.getEdge(6, 9), 1)



if __name__ == "__main__":
    #import sys;sys.argv = ['', 'Test.testName']
    unittest.main()